Index: src/org/armedbear/lisp/LispThread.java
===================================================================
--- src/org/armedbear/lisp/LispThread.java	(revision 12634)
+++ src/org/armedbear/lisp/LispThread.java	(working copy)
@@ -49,6 +49,7 @@
        new ConcurrentHashMap<Thread,LispThread>();
 
     LispObject threadValue = NIL;
+    LispObject[] threadValues = null;
 
     private static ThreadLocal<LispThread> threads = new ThreadLocal<LispThread>(){
         @Override
@@ -92,6 +93,15 @@
                     threadValue = funcall(wrapper,
                             new LispObject[] { fun },
                             LispThread.this);
+                    // Capture multiple values so they can be returned with
+                    // a join
+                    if (_values != null) {
+                        // allocate one extra for the join 
+                        // completed/interupted flag
+                        threadValues = new LispObject[_values.length+1];
+                        System.arraycopy(_values, 0, threadValues, 0,
+                                         _values.length);
+                    }
                 }
                 catch (ThreadDestroyed ignored) {
                       // Might happen.
@@ -945,15 +955,27 @@
             if (arg instanceof LispThread) {                
                 final LispThread joinedThread = (LispThread) arg;
                 final LispThread waitingThread = currentThread();
+                LispObject completedFlag = T;
                 try {
                     joinedThread.javaThread.join();
-                    return 
-                        waitingThread.setValues(joinedThread.threadValue, T);
                 } catch (InterruptedException e) {
                     waitingThread.processThreadInterrupts();
-                    return 
-                        waitingThread.setValues(joinedThread.threadValue, NIL);
+                    completedFlag = NIL;
                 }
+
+                if (joinedThread.threadValues != null) {
+                    waitingThread._values = joinedThread.threadValues;
+                    // threadValues already comes with an extra position for 
+                    // the completed flag
+                    waitingThread._values[waitingThread._values.length-1] 
+                      = completedFlag;
+                    return joinedThread.threadValue;
+                } else {
+                    return waitingThread.setValues
+                        (joinedThread.threadValue, completedFlag);
+                }
+                
+                
             } else {
                 return type_error(arg, Symbol.THREAD);
             } 
